/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.textcrypt.controller.implementation;

import com.frojasg1.general.dialogs.DialogsWrapper;
import com.frojasg1.general.progress.OperationCancellation;
import com.frojasg1.textcrypt.controller.TextcryptController;
import com.frojasg1.textcrypt.controller.TextcryptView;
import com.frojasg1.textcrypt.controller.threads.OpenEncryptedTextThread;
import com.frojasg1.textcrypt.controller.threads.SaveEncryptedTextThread;

/**
 *
 * @author axpe
 */
public class TextcryptController_imp implements TextcryptController
{
	protected TextcryptView _textcryptView = null;

	protected boolean _textModified = false;

	protected DialogsWrapper _dw = null;

	protected boolean _closeApplicationAfterSaving = false;

	public TextcryptController_imp( TextcryptView textcryptView,
									DialogsWrapper dw )
	{
		_textcryptView = textcryptView;
		_dw = dw;
	}

	@Override
	public void textCrypt_openFile()
	{
		boolean cont = true;

		if( _textModified )
		{
			Object[] options = new Object[]	{
				"Yes",
				"Cancel"
			};

			int answer = _dw.showOptionDialog( _textcryptView,
												"The text has been modified. If you continue, you will lose the changes. Continue anyway ?",
												"Warning",
												DialogsWrapper.CANCEL_OPTION,
												DialogsWrapper.WARNING_MESSAGE,
												options,
												null );

			if( ( answer == -1 ) || ( answer == 1 ) )		// cancel
			{
				cont = false;
			}
			else if( answer == 0 )		// yes
			{
				cont = true;
			}
		}

		if( cont )
		{
			if( ! _textcryptView.textCrypt_arePasswordEqual() )
			{
				_dw.showMessageDialog( _textcryptView,
										"Password are not equal. Solve it and try again",
										"Error",
										DialogsWrapper.ERROR_MESSAGE );
			}
			else
			{
				checkFileName();
				
				char[] password = _textcryptView.textCrypt_getPassword();

				OperationCancellation oc = new OperationCancellation( false );
				_textcryptView.textCrypt_setOperationCancellation( oc );
				_textcryptView.textCrypt_blockOpenAndSave();
				OpenEncryptedTextThread oett = new OpenEncryptedTextThread( this,
																			_textcryptView.textCrypt_getFileName(),
																			_textcryptView.textCrypt_getPassword(),
																			oc );
				oett.start();
			}
		}
	}

	@Override
	public void textCrypt_closing()
	{
		final int SAVE = 0;
		final int DO_NOT_SAVE = 1;
		final int CANCEL = 2;

		boolean okToClose = true;
		int answer = CANCEL;

		if( _textModified )
		{
			Object[] options = new Object[]	{
				"Yes",
				"No",
				"Cancel"
			};

			answer = _dw.showOptionDialog( _textcryptView,
											"The text has been modified. Do you want to save before exiting ?",
											"Warning",
											DialogsWrapper.YES_NO_CANCEL_OPTION,
											DialogsWrapper.WARNING_MESSAGE,
											options,
											null );

			if( ( answer == -1 ) || ( answer == CANCEL ) )		// cancel
			{
				okToClose = false;
			}
			else if( answer == SAVE )		// yes
			{
				okToClose = true;
			}
			else if( answer == DO_NOT_SAVE )		// no
			{
				okToClose = true;
			}
		}

		if( answer == SAVE )
		{
			_closeApplicationAfterSaving = true;

			textCrypt_saveFile();
		}
		else if( okToClose )
		{
			if( answer == DO_NOT_SAVE )
				_textModified = false;

			_textcryptView.textCrypt_closeWindow();
		}
	}

	@Override
	public void textCrypt_saveFile()
	{
		if( ! _textcryptView.textCrypt_arePasswordEqual() )
		{
			_dw.showMessageDialog( _textcryptView,
									"Password are not equal. Solve it and try again",
									"Error",
									DialogsWrapper.ERROR_MESSAGE );
		}
		else
		{
			checkFileName();
				
			char[] password = _textcryptView.textCrypt_getPassword();

			OperationCancellation oc = new OperationCancellation( false );
			_textcryptView.textCrypt_setOperationCancellation( oc );
			_textcryptView.textCrypt_blockOpenAndSave();
			SaveEncryptedTextThread sett = new SaveEncryptedTextThread( this,
																		_textcryptView.textCrypt_getFileName(),
																		_textcryptView.textCrypt_getPassword(),
																		_textcryptView.textCrypt_getStrength(),
																		_textcryptView.textCrypt_getText(),
																		oc );
			sett.start();
		}
	}

	@Override
	public void textCrypt_printText()
	{
		try
		{
			_textcryptView.textCrypt_printText();
		}
		catch( Throwable th )
		{
			th.printStackTrace();
			_dw.showMessageDialog( _textcryptView,
									"Printer exception: " + th,
									"Error",
									DialogsWrapper.ERROR_MESSAGE );
		}
	}

	@Override
	public void textCrypt_setTextModified()
	{
		_textModified = true;
	}

	@Override
	public void textCrypt_resetTextModified()
	{
		_textModified = false;
	}

	@Override
	public void textCrypt_fileSaved()
	{
		_textModified = false;
		_textcryptView.textCrypt_setOperationCancellation( null );
		_textcryptView.textCrypt_unblockOpenAndSave();

		if( _closeApplicationAfterSaving )
			_textcryptView.textCrypt_closeWindow();
	}

	@Override
	public void textCrypt_fileOpen( String text, int strength )
	{
//		_textModified = false;
		_textcryptView.textCrypt_setText( text );
		_textcryptView.textCrypt_setOperationCancellation( null );
		_textcryptView.textCrypt_unblockOpenAndSave();
		
		_textcryptView.textCrypt_setStrength( strength );

		// in case the document modified event is triggered.
		_textcryptView.textCrypt_invokeLaterResetTextModified();
	}

	@Override
	public void textCrypt_fileCancelled()
	{
		_textcryptView.textCrypt_setOperationCancellation( null );
		_dw.showMessageDialog( _textcryptView,
								"File cancelled",
								"Information",
								DialogsWrapper.INFORMATION_MESSAGE );
		_textcryptView.textCrypt_unblockOpenAndSave();

		_closeApplicationAfterSaving = false;
	}

	@Override
	public void textCrypt_fileError( String error )
	{
		_textcryptView.textCrypt_setOperationCancellation( null );
		_dw.showMessageDialog( _textcryptView,
								"File error: " + error,
								"Error",
								DialogsWrapper.ERROR_MESSAGE );
		_textcryptView.textCrypt_unblockOpenAndSave();

		_closeApplicationAfterSaving = false;
	}

	@Override
	public TextcryptView getView()
	{
		return( _textcryptView );
	}

	@Override
	public String textCrypt_chooseFile()
	{
		String fileName = _dw.showFileChooserDialog(_textcryptView,
													DialogsWrapper.OPEN,
													null,
													null);

		return( fileName );
	}

	protected void checkFileName()
	{
		if( _textcryptView.textCrypt_getFileName().length() == 0 )
		{
			String fileName = textCrypt_chooseFile();

			_textcryptView.textCrypt_setFileName( fileName );
		}
	}
}
